import pathlib
from typing import Any, Optional, Tuple
from unittest import mock

from click.testing import CliRunner

from gcs_migration.commands.create import create
from gcs_migration.models import MigrationPlan
from gcs_migration.parsers import GCSConfig, GridFTPOptionParser
from gcs_migration.util import AuthorizeSharing


def test_create(
    mock_get_transfer_api: Any,
    gcs_config: GCSConfig,
    gridftp_config_parser: GridFTPOptionParser,
    tmp_path: pathlib.Path,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
    host_endpoint_uuid: str,
) -> None:
    with mock.patch(
        "gcs_migration.commands.create.GCSConfig"
    ) as mock_gcs_config, mock.patch(
        "gcs_migration.commands.create.GridFTPOptionParser"
    ) as mock_gridftp_option_parser, mock.patch(
        "gcs_migration.commands.create.MigrationPlan"
    ) as mock_migration_plan, mock.patch.object(
        AuthorizeSharing, "map_and_authorize_sharing"
    ) as mock_map_and_authorize_sharing, mock.patch(
        "click.prompt", return_value=host_endpoint_uuid
    ):
        mock_gcs_config.return_value = gcs_config
        mock_gridftp_option_parser.return_value = gridftp_config_parser
        mock_map_and_authorize_sharing.return_value = authorization_response_success
        ret_migration_plan = MigrationPlan(tmp_path)
        mock_migration_plan.return_value = ret_migration_plan

        runner = CliRunner()
        result = runner.invoke(create)
        assert result.exit_code == 0
        loaded_plan = MigrationPlan.load(tmp_path)

        assert (
            loaded_plan.gsi_authz_configuration
            == migration_plan_cmd_create.gsi_authz_configuration
        )
        assert loaded_plan.storage_gateway == migration_plan_cmd_create.storage_gateway
        assert (
            loaded_plan.mapped_collection == migration_plan_cmd_create.mapped_collection
        )
        assert (
            loaded_plan.guest_collections == migration_plan_cmd_create.guest_collections
        )
        assert loaded_plan == migration_plan_cmd_create
        assert loaded_plan == ret_migration_plan


def test_requires_logged_in(mock_not_logged_in: None) -> None:
    runner = CliRunner()
    result = runner.invoke(create, catch_exceptions=False)
    assert "you will have to login" in result.output
